clc; clear; % Copyright: Junchao Wang 2017 Summer | first run 6/8/2017

while 1
    unitLength = 500; % um
    channelWidth = 50;
    letLength = 250;
    
    % random design domain begin
    chipName = num2str(round(rand * 10000000000000000))
    
    mainInletPosition = getRand(0, unitLength - channelWidth * 2);
    mainOutletPosition = getRand(0, unitLength - channelWidth * 2);
    
    % circle
    margin = 0;
    diameter = 100;
    maxNum = 10;
    coordinateSet = [];
    for i = 1: maxNum
        coordinateSet = [coordinateSet; ...
            getRand(margin, unitLength) ...
            getRand(margin, unitLength - margin) ...
            getRand(0.3 * diameter, diameter)];
    end

    spot = [0 0; 0 unitLength; mainInletPosition unitLength;...
        mainInletPosition (unitLength + letLength); ...
        (mainInletPosition + channelWidth) (unitLength + letLength);...
        (mainInletPosition + channelWidth * 2) (unitLength + letLength);...
        (mainInletPosition + channelWidth * 2) unitLength;...
        unitLength unitLength; unitLength 0;...
        (mainOutletPosition + channelWidth * 2) 0; ...
        (mainOutletPosition + channelWidth * 2) (- letLength);...
        (mainOutletPosition + channelWidth) (- letLength); ...
        mainOutletPosition (- letLength);...
        mainOutletPosition 0; 0 0];
    % comsol begin
    geomd = 2;
    modelfilepath = 'C:\simulation';
    out = modelCreateFirstStep(geomd, modelfilepath);

    % design domain
    out = CreatePolygon(out, 'designDomain', spot);
    inlet_one = intersect(getBoundaryNumber(out, [(mainInletPosition + channelWidth) (unitLength + letLength)]), getBoundaryNumber(out, [(mainInletPosition) (unitLength + letLength)]));
    inlet_two = intersect(getBoundaryNumber(out, [(mainInletPosition + channelWidth) (unitLength + letLength)]), getBoundaryNumber(out, [(mainInletPosition + channelWidth *2) (unitLength + letLength)]));

    outlet_one = intersect(getBoundaryNumber(out, [(mainOutletPosition + channelWidth) (-letLength)]), getBoundaryNumber(out, [(mainOutletPosition) (-letLength)]));
    outlet_two = intersect(getBoundaryNumber(out, [(mainOutletPosition + channelWidth) (-letLength)]), getBoundaryNumber(out, [(mainOutletPosition+ channelWidth *2) (-letLength)]));


    % inlet 9, 15 outlet 5, 11
    FlowRate = [0.001 0.001];
    % geo not done yet
    inlet = [inlet_one inlet_two];
    outlet = [outlet_one outlet_two];
    out = AddWaterAsMaterial(out);
    out = AddLaminarFlow(out, inlet, outlet, FlowRate);
    out = AddTransportOfDilutedSpecies(out, inlet, outlet);

    % design domain % build subtractions
    total = numel(coordinateSet) / 3;
    subSet = {};
    for i = 1: total
        name = strcat('sub', num2str(i));
        out = CreateCycle(out, name, 'center', coordinateSet(i, 3), coordinateSet(i, 1),...
            coordinateSet(i, 2), 360, 0);
        subSet = [subSet name];
    end
    out = CreateDifference(out, 'finalChannel', {'designDomain'}, subSet);

    out = CreateTriMesh(out);
    % 
    try
        out = StudyOneForLaminar(out);
        laminarWorks = 1;
    catch
        laminarWorks = 0;
    end
    % 
    if laminarWorks
        try
            out = StudyTwoForTransportOfDilutedSpecies(out);
            dilutedSpeciesWorks = 1;
            in1conc = mphglobal(out,'chds.c0avg_c_in1','dataset','dset2');
            in2conc = mphglobal(out,'chds.c0avg_c_in2','dataset','dset2');
            out1conc = mphglobal(out,'chds.c0avg_c_out1','dataset','dset2');
            out2conc = mphglobal(out,'chds.c0avg_c_out2','dataset','dset2');
            res = [1 in1conc out1conc in2conc out2conc getPressureData(out)]
        catch
            dilutedSpeciesWorks = 0;
            in1conc = mphglobal(out,'chds.c0avg_c_in1','dataset','dset2');
            in2conc = mphglobal(out,'chds.c0avg_c_in2','dataset','dset2');
            out1conc = mphglobal(out,'chds.c0avg_c_out1','dataset','dset2');
            out2conc = mphglobal(out,'chds.c0avg_c_out2','dataset','dset2');
            res = [0 in1conc out1conc in2conc out2conc getPressureData(out)]
        end
    else
        res = -1;
    end

    if res ~= -1
        random_simulation_result = [random_simulation_result; {chipName res coordinateSet mainInletPosition mainOutletPosition}];
        save('random_simulation_result.mat', 'random_simulation_result')
    end
    break
end


